@@ -54,20 +54,27 @@ import asyncio
 import json
 import logging
 import os
+import urllib.request
+from asyncio.tasks import Task
 from typing import Any, Dict, List, Optional, Union
 
 import aiohttp
+import nest_asyncio
 import pkg_resources
 import requests
 import tqdm
 import tqdm.notebook as tqdm_notebook
 
+from nucleus.url_utils import sanitize_string_args
+
 from .annotation import (
     BoxAnnotation,
+    CuboidAnnotation,
+    Point,
+    Point3D,
     PolygonAnnotation,
     Segment,
     SegmentationAnnotation,
-    Point,
 )
 from .constants import (
     ANNOTATION_METADATA_SCHEMA_KEY,
@@ -77,14 +84,22 @@ from .constants import (
     DATASET_ID_KEY,
     DATASET_ITEM_IDS_KEY,
     DEFAULT_NETWORK_TIMEOUT_SEC,
+    EMBEDDING_DIMENSION_KEY,
     EMBEDDINGS_URL_KEY,
     ERROR_ITEMS,
     ERROR_PAYLOAD,
     ERRORS_KEY,
+    JOB_ID_KEY,
+    JOB_LAST_KNOWN_STATUS_KEY,
+    JOB_TYPE_KEY,
+    JOB_CREATION_TIME_KEY,
     IMAGE_KEY,
     IMAGE_URL_KEY,
+    INDEX_CONTINUOUS_ENABLE_KEY,
     ITEM_METADATA_SCHEMA_KEY,
     ITEMS_KEY,
+    KEEP_HISTORY_KEY,
+    MESSAGE_KEY,
     MODEL_RUN_ID_KEY,
     NAME_KEY,
     NUCLEUS_ENDPOINT,
@@ -96,7 +111,7 @@ from .constants import (
     UPDATE_KEY,
 )
 from .dataset import Dataset
-from .dataset_item import DatasetItem
+from .dataset_item import DatasetItem, CameraParams, Quaternion
 from .errors import (
     DatasetItemRetrievalError,
     ModelCreationError,
@@ -104,6 +119,7 @@ from .errors import (
     NotFoundError,
     NucleusAPIError,
 )
+from .job import AsyncJob
 from .model import Model
 from .model_run import ModelRun
 from .payload_constructor import (
@@ -115,11 +131,13 @@ from .payload_constructor import (
 )
 from .prediction import (
     BoxPrediction,
+    CuboidPrediction,
     PolygonPrediction,
     SegmentationPrediction,
 )
 from .slice import Slice
 from .upload_response import UploadResponse
+from .scene import Frame, LidarScene
 
 # pylint: disable=E1101
 # TODO: refactor to reduce this file to under 1000 lines.
@@ -176,11 +194,11 @@ class NucleusClient:
 
         return [
             Model(
-                model["id"],
-                model["name"],
-                model["ref_id"],
-                model["metadata"],
-                self,
+                model_id=model["id"],
+                name=model["name"],
+                reference_id=model["ref_id"],
+                metadata=model["metadata"] or None,
+                client=self,
             )
             for model in model_objects["models"]
         ]
@@ -192,6 +210,26 @@ class NucleusClient:
         """
         return self.make_request({}, "dataset/", requests.get)
 
+    def list_jobs(
+        self, show_completed=None, date_limit=None
+    ) -> List[AsyncJob]:
+        """
+        Lists jobs for user.
+        :return: jobs
+        """
+        payload = {show_completed: show_completed, date_limit: date_limit}
+        job_objects = self.make_request(payload, "jobs/", requests.get)
+        return [
+            AsyncJob(
+                job_id=job[JOB_ID_KEY],
+                job_last_known_status=job[JOB_LAST_KNOWN_STATUS_KEY],
+                job_type=job[JOB_TYPE_KEY],
+                job_creation_time=job[JOB_CREATION_TIME_KEY],
+                client=self,
+            )
+            for job in job_objects
+        ]
+
     def get_dataset_items(self, dataset_id) -> List[DatasetItem]:
         """
         Gets all the dataset items inside your repo as a json blob.
@@ -226,6 +264,19 @@ class NucleusClient:
         """
         return Dataset(dataset_id, self)
 
+    def get_model(self, model_id: str) -> Model:
+        """
+        Fetched a model for a given id
+        :param model_id: internally controlled dataset_id
+        :return: model
+        """
+        payload = self.make_request(
+            payload={},
+            route=f"model/{model_id}",
+            requests_command=requests.get,
+        )
+        return Model.from_json(payload=payload, client=self)
+
     def get_model_run(self, model_run_id: str, dataset_id: str) -> ModelRun:
         """
         Fetches a model_run for given id
@@ -298,6 +349,7 @@ class NucleusClient:
         """
         return self.make_request({}, f"dataset/{dataset_id}", requests.delete)
 
+    @sanitize_string_args
     def delete_dataset_item(
         self, dataset_id: str, item_id: str = None, reference_id: str = None
     ) -> dict:
@@ -366,28 +418,33 @@ class NucleusClient:
 
         agg_response = UploadResponse(json={DATASET_ID_KEY: dataset_id})
 
-        tqdm_local_batches = self.tqdm_bar(local_batches)
-
-        tqdm_remote_batches = self.tqdm_bar(remote_batches)
-
         async_responses: List[Any] = []
 
-        for batch in tqdm_local_batches:
-            payload = construct_append_payload(batch, update)
-            responses = self._process_append_requests_local(
-                dataset_id, payload, update
+        if local_batches:
+            tqdm_local_batches = self.tqdm_bar(
+                local_batches, desc="Local file batches"
             )
-            async_responses.extend(responses)
 
-        for batch in tqdm_remote_batches:
-            payload = construct_append_payload(batch, update)
-            responses = self._process_append_requests(
-                dataset_id=dataset_id,
-                payload=payload,
-                update=update,
-                batch_size=batch_size,
+            for batch in tqdm_local_batches:
+                payload = construct_append_payload(batch, update)
+                responses = self._process_append_requests_local(
+                    dataset_id, payload, update
+                )
+                async_responses.extend(responses)
+
+        if remote_batches:
+            tqdm_remote_batches = self.tqdm_bar(
+                remote_batches, desc="Remote file batches"
             )
-            async_responses.extend(responses)
+            for batch in tqdm_remote_batches:
+                payload = construct_append_payload(batch, update)
+                responses = self._process_append_requests(
+                    dataset_id=dataset_id,
+                    payload=payload,
+                    update=update,
+                    batch_size=batch_size,
+                )
+                async_responses.extend(responses)
 
         for response in async_responses:
             agg_response.update_response(response)
@@ -402,6 +459,8 @@ class NucleusClient:
         local_batch_size: int = 10,
     ):
         def get_files(batch):
+            for item in batch:
+                item[UPDATE_KEY] = update
             request_payload = [
                 (
                     ITEMS_KEY,
@@ -434,14 +493,20 @@ class NucleusClient:
             files_per_request.append(get_files(batch))
             payload_items.append(batch)
 
-        loop = asyncio.get_event_loop()
-        responses = loop.run_until_complete(
-            self.make_many_files_requests_asynchronously(
-                files_per_request,
-                f"dataset/{dataset_id}/append",
-            )
+        future = self.make_many_files_requests_asynchronously(
+            files_per_request,
+            f"dataset/{dataset_id}/append",
         )
 
+        try:
+            loop = asyncio.get_event_loop()
+        except RuntimeError:  # no event loop running:
+            loop = asyncio.new_event_loop()
+            responses = loop.run_until_complete(future)
+        else:
+            nest_asyncio.apply(loop)
+            return loop.run_until_complete(future)
+
         def close_files(request_items):
             for item in request_items:
                 # file buffer in location [1][1]
@@ -553,7 +618,12 @@ class NucleusClient:
         self,
         dataset_id: str,
         annotations: List[
-            Union[BoxAnnotation, PolygonAnnotation, SegmentationAnnotation]
+            Union[
+                BoxAnnotation,
+                PolygonAnnotation,
+                CuboidAnnotation,
+                SegmentationAnnotation,
+            ]
         ],
         update: bool,
         batch_size: int = 5000,
@@ -561,11 +631,10 @@ class NucleusClient:
         """
         Uploads ground truth annotations for a given dataset.
         :param dataset_id: id of the dataset
-        :param annotations: List[Union[BoxAnnotation, PolygonAnnotation]]
+        :param annotations: List[Union[BoxAnnotation, PolygonAnnotation, CuboidAnnotation, SegmentationAnnotation]]
         :param update: whether to update or ignore conflicting annotations
         :return: {"dataset_id: str, "annotations_processed": int}
         """
-
         # Split payload into segmentations and Box/Polygon
         segmentations = [
             ann
@@ -706,14 +775,19 @@ class NucleusClient:
         self,
         model_run_id: str,
         annotations: List[
-            Union[BoxPrediction, PolygonPrediction, SegmentationPrediction]
+            Union[
+                BoxPrediction,
+                PolygonPrediction,
+                CuboidPrediction,
+                SegmentationPrediction,
+            ]
         ],
         update: bool,
         batch_size: int = 5000,
     ):
         """
         Uploads model outputs as predictions for a model_run. Returns info about the upload.
-        :param annotations: List[Union[BoxPrediction, PolygonPrediction]],
+        :param annotations: List[Union[BoxPrediction, PolygonPrediction, CuboidPrediction, SegmentationPrediction]],
         :param update: bool
         :return:
         {
@@ -855,6 +929,7 @@ class NucleusClient:
             {}, f"modelRun/{model_run_id}/info", requests.get
         )
 
+    @sanitize_string_args
     def dataitem_ref_id(self, dataset_id: str, reference_id: str):
         """
         :param dataset_id: internally controlled dataset id
@@ -865,6 +940,7 @@ class NucleusClient:
             {}, f"dataset/{dataset_id}/refloc/{reference_id}", requests.get
         )
 
+    @sanitize_string_args
     def predictions_ref_id(self, model_run_id: str, ref_id: str):
         """
         Returns Model Run info For Dataset Item by model_run_id and item reference_id.
@@ -872,7 +948,7 @@ class NucleusClient:
         :param reference_id: reference_id of a dataset item.
         :return:
         {
-            "annotations": List[BoxPrediction],
+            "annotations": List[Union[BoxPrediction, PolygonPrediction, CuboidPrediction, SegmentationPrediction]],
         }
         """
         return self.make_request(
@@ -897,7 +973,7 @@ class NucleusClient:
         :param i: absolute number of Dataset Item for a dataset corresponding to the model run.
         :return:
         {
-            "annotations": List[BoxPrediction],
+            "annotations": List[Union[BoxPrediction, PolygonPrediction, CuboidPrediction, SegmentationPrediction]],
         }
         """
         return self.make_request(
@@ -926,7 +1002,7 @@ class NucleusClient:
         :param dataset_item_id: dataset_item_id of a dataset item.
         :return:
         {
-            "annotations": List[BoxPrediction],
+            "annotations": List[Union[BoxPrediction, PolygonPrediction, CuboidPrediction, SegmentationPrediction]],
         }
         """
         return self.make_request(
@@ -1010,6 +1086,28 @@ class NucleusClient:
         )
         return response
 
+    def delete_annotations(
+        self, dataset_id: str, reference_ids: list = None, keep_history=False
+    ) -> dict:
+        """
+        This endpoint deletes annotations.
+
+        :param
+        slice_id: id of the slice
+
+        :return:
+        {}
+        """
+        payload = {KEEP_HISTORY_KEY: keep_history}
+        if reference_ids:
+            payload[REFERENCE_IDS_KEY] = reference_ids
+        response = self.make_request(
+            payload,
+            f"annotation/{dataset_id}",
+            requests_command=requests.delete,
+        )
+        return response
+
     def append_to_slice(
         self,
         slice_id: str,
@@ -1057,6 +1155,16 @@ class NucleusClient:
         )
         return response[AUTOTAGS_KEY] if AUTOTAGS_KEY in response else response
 
+    def delete_autotag(self, autotag_id: str) -> dict:
+        """
+        Deletes an autotag based on autotagId.
+        Returns an empty payload where response status `200` indicates
+        the autotag has been successfully deleted.
+        :param autotag_id: id of the autotag to delete.
+        :return: {}
+        """
+        return self.make_request({}, f"autotag/{autotag_id}", requests.delete)
+
     def delete_model(self, model_id: str) -> dict:
         """
         This endpoint deletes the specified model, along with all
@@ -1075,9 +1183,23 @@ class NucleusClient:
         )
         return response
 
-    def create_custom_index(self, dataset_id: str, embeddings_url: str):
+    def create_custom_index(
+        self, dataset_id: str, embeddings_urls: list, embedding_dim: int
+    ):
+        """
+        Creates a custom index for a given dataset, which will then be used
+        for autotag and similarity search.
+
+        :param
+        dataset_id: id of dataset that the custom index is being added to.
+        embeddings_urls: list of urls, each of which being a json mapping dataset_item_id -> embedding vector
+        embedding_dim: the dimension of the embedding vectors, must be consistent for all embedding vectors in the index.
+        """
         return self.make_request(
-            {EMBEDDINGS_URL_KEY: embeddings_url},
+            {
+                EMBEDDINGS_URL_KEY: embeddings_urls,
+                EMBEDDING_DIMENSION_KEY: embedding_dim,
+            },
             f"indexing/{dataset_id}",
             requests_command=requests.post,
         )
@@ -1096,6 +1218,37 @@ class NucleusClient:
             requests_command=requests.delete,
         )
 
+    def set_continuous_indexing(self, dataset_id: str, enable: bool = True):
+        """
+        Sets continuous indexing for a given dataset, which will automatically generate embeddings whenever
+        new images are uploaded. This endpoint is currently only enabled for enterprise customers.
+        Please reach out to nucleus@scale.com if you wish to learn more.
+
+        :param
+        dataset_id: id of dataset that continuous indexing is being toggled for
+        enable: boolean, sets whether we are enabling or disabling continuous indexing. The default behavior is to enable.
+        """
+        return self.make_request(
+            {INDEX_CONTINUOUS_ENABLE_KEY: enable},
+            f"indexing/{dataset_id}/setContinuous",
+            requests_command=requests.post,
+        )
+
+    def create_image_index(self, dataset_id: str):
+        """
+        Starts generating embeddings for images that don't have embeddings in a given dataset. These embeddings will
+        be used for autotag and similarity search. This endpoint is currently only enabled for enterprise customers.
+        Please reach out to nucleus@scale.com if you wish to learn more.
+
+        :param
+        dataset_id: id of dataset for generating embeddings on.
+        """
+        return self.make_request(
+            {},
+            f"indexing/{dataset_id}/internal/image",
+            requests_command=requests.post,
+        )
+
     def make_request(
         self, payload: dict, route: str, requests_command=requests.post
     ) -> dict:
