@@ -2,13 +2,15 @@ import json
 from dataclasses import dataclass
 from enum import Enum
 from typing import Dict, List, Optional, Sequence, Union
-from nucleus.dataset_item import is_local_path
+from urllib.parse import urlparse
 
 from .constants import (
     ANNOTATION_ID_KEY,
     ANNOTATIONS_KEY,
     BOX_TYPE,
+    CUBOID_TYPE,
     DATASET_ITEM_ID_KEY,
+    DIMENSIONS_KEY,
     GEOMETRY_KEY,
     HEIGHT_KEY,
     INDEX_KEY,
@@ -18,12 +20,15 @@ from .constants import (
     MASK_URL_KEY,
     METADATA_KEY,
     POLYGON_TYPE,
+    POSITION_KEY,
     REFERENCE_ID_KEY,
     TYPE_KEY,
     VERTICES_KEY,
     WIDTH_KEY,
     X_KEY,
+    YAW_KEY,
     Y_KEY,
+    Z_KEY,
 )
 
 
@@ -32,7 +37,9 @@ class Annotation:
     item_id: Optional[str] = None
 
     def _check_ids(self):
-        if bool(self.reference_id) == bool(self.item_id):
+        if self.reference_id and self.item_id:
+            self.item_id = None  # Prefer reference id to item id.
+        if not (self.reference_id or self.item_id):
             raise Exception(
                 "You must specify either a reference_id or an item_id for an annotation."
             )
@@ -43,6 +50,8 @@ class Annotation:
             return BoxAnnotation.from_json(payload)
         elif payload.get(TYPE_KEY, None) == POLYGON_TYPE:
             return PolygonAnnotation.from_json(payload)
+        elif payload.get(TYPE_KEY, None) == CUBOID_TYPE:
+            return CuboidAnnotation.from_json(payload)
         else:
             return SegmentationAnnotation.from_json(payload)
 
@@ -125,6 +134,7 @@ class SegmentationAnnotation(Annotation):
 class AnnotationTypes(Enum):
     BOX = BOX_TYPE
     POLYGON = POLYGON_TYPE
+    CUBOID = CUBOID_TYPE
 
 
 @dataclass  # pylint: disable=R0902
@@ -187,6 +197,20 @@ class Point:
         return {X_KEY: self.x, Y_KEY: self.y}
 
 
+@dataclass
+class Point3D:
+    x: float
+    y: float
+    z: float
+
+    @classmethod
+    def from_json(cls, payload: Dict[str, float]):
+        return cls(payload[X_KEY], payload[Y_KEY], payload[Z_KEY])
+
+    def to_payload(self) -> dict:
+        return {X_KEY: self.x, Y_KEY: self.y, Z_KEY: self.z}
+
+
 @dataclass
 class PolygonAnnotation(Annotation):
     label: str
@@ -241,12 +265,65 @@ class PolygonAnnotation(Annotation):
         return payload
 
 
-def check_all_annotation_paths_remote(
+@dataclass  # pylint: disable=R0902
+class CuboidAnnotation(Annotation):  # pylint: disable=R0902
+    label: str
+    position: Point3D
+    dimensions: Point3D
+    yaw: float
+    reference_id: Optional[str] = None
+    item_id: Optional[str] = None
+    annotation_id: Optional[str] = None
+    metadata: Optional[Dict] = None
+
+    def __post_init__(self):
+        self._check_ids()
+        self.metadata = self.metadata if self.metadata else {}
+
+    @classmethod
+    def from_json(cls, payload: dict):
+        geometry = payload.get(GEOMETRY_KEY, {})
+        return cls(
+            label=payload.get(LABEL_KEY, 0),
+            position=Point3D.from_json(geometry.get(POSITION_KEY, {})),
+            dimensions=Point3D.from_json(geometry.get(DIMENSIONS_KEY, {})),
+            yaw=geometry.get(YAW_KEY, 0),
+            reference_id=payload.get(REFERENCE_ID_KEY, None),
+            item_id=payload.get(DATASET_ITEM_ID_KEY, None),
+            annotation_id=payload.get(ANNOTATION_ID_KEY, None),
+            metadata=payload.get(METADATA_KEY, {}),
+        )
+
+    def to_payload(self) -> dict:
+        payload = {
+            LABEL_KEY: self.label,
+            TYPE_KEY: CUBOID_TYPE,
+            GEOMETRY_KEY: {
+                POSITION_KEY: self.position.to_payload(),
+                DIMENSIONS_KEY: self.dimensions.to_payload(),
+                YAW_KEY: self.yaw,
+            },
+        }
+        if self.reference_id:
+            payload[REFERENCE_ID_KEY] = self.reference_id
+        if self.annotation_id:
+            payload[ANNOTATION_ID_KEY] = self.annotation_id
+        if self.metadata:
+            payload[METADATA_KEY] = self.metadata
+
+        return payload
+
+
+def is_local_path(path: str) -> bool:
+    return urlparse(path).scheme not in {"https", "http", "s3", "gs"}
+
+
+def check_all_mask_paths_remote(
     annotations: Sequence[Union[Annotation]],
 ):
     for annotation in annotations:
         if hasattr(annotation, MASK_URL_KEY):
             if is_local_path(getattr(annotation, MASK_URL_KEY)):
                 raise ValueError(
-                    f"Found an annotation with a local path, which cannot be uploaded asynchronously. Use a remote path instead. {annotation}"
+                    f"Found an annotation with a local path, which is not currently supported. Use a remote path instead. {annotation}"
                 )
