@@ -7,12 +7,16 @@ BOX_TYPE = "box"
 POLYGON_TYPE = "polygon"
 MASK_TYPE = "mask"
 SEGMENTATION_TYPE = "segmentation"
-ANNOTATION_TYPES = (BOX_TYPE, POLYGON_TYPE, SEGMENTATION_TYPE)
+CUBOID_TYPE = "cuboid"
+ANNOTATION_TYPES = (BOX_TYPE, POLYGON_TYPE, SEGMENTATION_TYPE, CUBOID_TYPE)
 ANNOTATION_UPDATE_KEY = "update"
 AUTOTAGS_KEY = "autotags"
 EXPORTED_ROWS = "exportedRows"
+CAMERA_PARAMS_KEY = "camera_params"
 CLASS_PDF_KEY = "class_pdf"
 CONFIDENCE_KEY = "confidence"
+CX_KEY = "cx"
+CY_KEY = "cy"
 DATASET_ID_KEY = "dataset_id"
 DATASET_ITEM_IDS_KEY = "dataset_item_ids"
 DATASET_ITEM_ID_KEY = "dataset_item_id"
@@ -22,22 +26,37 @@ DATASET_NAME_KEY = "name"
 DATASET_SLICES_KEY = "slice_ids"
 DEFAULT_ANNOTATION_UPDATE_MODE = False
 DEFAULT_NETWORK_TIMEOUT_SEC = 120
-EMBEDDINGS_URL_KEY = "embeddings_url"
+DIMENSIONS_KEY = "dimensions"
+EMBEDDINGS_URL_KEY = "embeddings_urls"
+EMBEDDING_DIMENSION_KEY = "embedding_dimension"
 ERRORS_KEY = "errors"
 ERROR_CODES = "error_codes"
 ERROR_ITEMS = "upload_errors"
 ERROR_PAYLOAD = "error_payload"
+FRAMES_KEY = "frames"
+FX_KEY = "fx"
+FY_KEY = "fy"
 GEOMETRY_KEY = "geometry"
+HEADING_KEY = "heading"
 HEIGHT_KEY = "height"
 IGNORED_ITEMS = "ignored_items"
 IMAGE_KEY = "image"
+IMAGE_LOCATION_KEY = "image_location"
 IMAGE_URL_KEY = "image_url"
 INDEX_KEY = "index"
+INDEX_CONTINUOUS_ENABLE_KEY = "enable"
 ITEMS_KEY = "items"
 ITEM_ID_KEY = "item_id"
 ITEM_KEY = "item"
+ITEMS_KEY = "items"
 ITEM_METADATA_SCHEMA_KEY = "item_metadata_schema"
 JOB_ID_KEY = "job_id"
+KEEP_HISTORY_KEY = "keep_history"
+LENGTH_KEY = "length"
+JOB_STATUS_KEY = "job_status"
+JOB_LAST_KNOWN_STATUS_KEY = "job_last_known_status"
+JOB_TYPE_KEY = "job_type"
+JOB_CREATION_TIME_KEY = "job_creation_time"
 LABEL_KEY = "label"
 MASK_URL_KEY = "mask_url"
 MESSAGE_KEY = "message"
@@ -47,12 +66,18 @@ MODEL_RUN_ID_KEY = "model_run_id"
 NAME_KEY = "name"
 NEW_ITEMS = "new_items"
 NUCLEUS_ENDPOINT = "https://api.scale.com/v1/nucleus"
+NUM_SENSORS_KEY = "num_sensors"
 ORIGINAL_IMAGE_URL_KEY = "original_image_url"
+POINTCLOUD_KEY = "pointcloud"
+POINTCLOUD_LOCATION_KEY = "pointcloud_location"
+POINTCLOUD_URL_KEY = "pointcloud_url"
+POSITION_KEY = "position"
 PREDICTIONS_IGNORED_KEY = "predictions_ignored"
 PREDICTIONS_PROCESSED_KEY = "predictions_processed"
 REFERENCE_IDS_KEY = "reference_ids"
 REFERENCE_ID_KEY = "reference_id"
 REQUEST_ID_KEY = "requestId"
+SCENES_KEY = "scenes"
 SEGMENTATIONS_KEY = "segmentations"
 SLICE_ID_KEY = "slice_id"
 STATUS_CODE_KEY = "status_code"
@@ -61,7 +86,12 @@ SUCCESS_STATUS_CODES = [200, 201, 202]
 TYPE_KEY = "type"
 UPDATED_ITEMS = "updated_items"
 UPDATE_KEY = "update"
+UPLOAD_TO_SCALE_KEY = "upload_to_scale"
+URL_KEY = "url"
 VERTICES_KEY = "vertices"
 WIDTH_KEY = "width"
+YAW_KEY = "yaw"
+W_KEY = "w"
 X_KEY = "x"
 Y_KEY = "y"
+Z_KEY = "z"
