@@ -3,13 +3,17 @@ from typing import Any, Dict, List, Optional, Union
 import requests
 
 from nucleus.job import AsyncJob
+from nucleus.url_utils import sanitize_string_args
 from nucleus.utils import (
     convert_export_payload,
     format_dataset_item_response,
     serialize_and_write_to_presigned_url,
 )
 
-from .annotation import Annotation, check_all_annotation_paths_remote
+from .annotation import (
+    Annotation,
+    check_all_mask_paths_remote,
+)
 from .constants import (
     DATASET_ITEM_IDS_KEY,
     DATASET_LENGTH_KEY,
@@ -18,7 +22,6 @@ from .constants import (
     DATASET_SLICES_KEY,
     DEFAULT_ANNOTATION_UPDATE_MODE,
     EXPORTED_ROWS,
-    JOB_ID_KEY,
     NAME_KEY,
     REFERENCE_IDS_KEY,
     REQUEST_ID_KEY,
@@ -29,10 +32,14 @@ from .dataset_item import (
     check_all_paths_remote,
     check_for_duplicate_reference_ids,
 )
-from .payload_constructor import construct_model_run_creation_payload
-
+from .scene import LidarScene, check_all_scene_paths_remote
+from .payload_constructor import (
+    construct_append_scenes_payload,
+    construct_model_run_creation_payload,
+)
 
 WARN_FOR_LARGE_UPLOAD = 50000
+WARN_FOR_LARGE_SCENES_UPLOAD = 5
 
 
 class Dataset:
@@ -79,6 +86,7 @@ class Dataset:
     def items(self) -> List[DatasetItem]:
         return self._client.get_dataset_items(self.id)
 
+    @sanitize_string_args
     def autotag_scores(self, autotag_name, for_scores_greater_than=0):
         """Export the autotag scores above a threshold, largest scores first.
 
@@ -163,9 +171,9 @@ class Dataset:
             "ignored_items": int,
         }
         """
-        if asynchronous:
-            check_all_annotation_paths_remote(annotations)
+        check_all_mask_paths_remote(annotations)
 
+        if asynchronous:
             request_id = serialize_and_write_to_presigned_url(
                 annotations, self.id, self._client
             )
@@ -173,8 +181,7 @@ class Dataset:
                 payload={REQUEST_ID_KEY: request_id, UPDATE_KEY: update},
                 route=f"dataset/{self.id}/annotate?async=1",
             )
-
-            return AsyncJob(response[JOB_ID_KEY], self._client)
+            return AsyncJob.from_json(response, self._client)
 
         return self._client.annotate_dataset(
             self.id, annotations, update=update, batch_size=batch_size
@@ -193,16 +200,16 @@ class Dataset:
 
     def append(
         self,
-        dataset_items: List[DatasetItem],
+        items: Union[List[DatasetItem], List[LidarScene]],
         update: Optional[bool] = False,
         batch_size: Optional[int] = 20,
         asynchronous=False,
     ) -> Union[dict, AsyncJob]:
         """
-        Appends images with metadata (dataset items) to the dataset. Overwrites images on collision if forced.
+        Appends images with metadata (dataset items) or scenes to the dataset. Overwrites images on collision if forced.
 
         Parameters:
-        :param dataset_items: items to upload
+        :param items: items to upload
         :param update: if True overwrites images and metadata on collision
         :param batch_size: batch parameter for long uploads
         :param aynchronous: if True, return a job object representing asynchronous ingestion job.
@@ -214,6 +221,17 @@ class Dataset:
             'ignored_items': int,
         }
         """
+        dataset_items = [
+            item for item in items if isinstance(item, DatasetItem)
+        ]
+        scenes = [item for item in items if isinstance(item, LidarScene)]
+        if dataset_items and scenes:
+            raise Exception(
+                "You must append either DatasetItems or Scenes to the dataset."
+            )
+        if scenes:
+            return self.append_scenes(scenes, update, asynchronous)
+
         check_for_duplicate_reference_ids(dataset_items)
 
         if len(dataset_items) > WARN_FOR_LARGE_UPLOAD and not asynchronous:
@@ -233,7 +251,7 @@ class Dataset:
                 payload={REQUEST_ID_KEY: request_id, UPDATE_KEY: update},
                 route=f"dataset/{self.id}/append?async=1",
             )
-            return AsyncJob(response["job_id"], self._client)
+            return AsyncJob.from_json(response, self._client)
 
         return self._client.populate_dataset(
             self.id,
@@ -242,6 +260,57 @@ class Dataset:
             batch_size=batch_size,
         )
 
+    def append_scenes(
+        self,
+        scenes: List[LidarScene],
+        update: Optional[bool] = False,
+        asynchronous: Optional[bool] = False,
+    ) -> Union[dict, AsyncJob]:
+        """
+        Appends scenes with given frames (containing pointclouds and optional images) to the dataset
+
+        Parameters:
+        :param scenes: scenes to upload
+        :param update: if True, overwrite scene on collision
+        :param asynchronous: if True, return a job object representing asynchronous ingestion job
+        :return:
+        {
+            'dataset_id': str,
+            'new_scenes': int,
+            'ignored_scenes': int,
+            'scenes_errored': int,
+            'errors': List[str],
+        }
+        """
+        for scene in scenes:
+            scene.validate()
+
+        if len(scenes) > WARN_FOR_LARGE_SCENES_UPLOAD and not asynchronous:
+            print(
+                "Tip: for large uploads, get faster performance by importing your data "
+                "into Nucleus directly from a cloud storage provider. See "
+                "https://dashboard.scale.com/nucleus/docs/api?language=python#guide-for-large-ingestions"
+                " for details."
+            )
+
+        if asynchronous:
+            check_all_scene_paths_remote(scenes)
+            request_id = serialize_and_write_to_presigned_url(
+                scenes, self.id, self._client
+            )
+            response = self._client.make_request(
+                payload={REQUEST_ID_KEY: request_id, UPDATE_KEY: update},
+                route=f"{self.id}/upload_scenes?async=1",
+            )
+            return AsyncJob.from_json(response, self._client)
+
+        payload = construct_append_scenes_payload(scenes, update)
+        response = self._client.make_request(
+            payload=payload,
+            route=f"{self.id}/upload_scenes",
+        )
+        return response
+
     def iloc(self, i: int) -> dict:
         """
         Returns Dataset Item Info By Dataset Item Number.
@@ -249,7 +318,7 @@ class Dataset:
         :return:
         {
             "item": DatasetItem,
-            "annotations": List[Union[BoxAnnotation, PolygonAnnotation]],
+            "annotations": List[Union[BoxAnnotation, PolygonAnnotation, CuboidAnnotation, SegmentationAnnotation]],
         }
         """
         response = self._client.dataitem_iloc(self.id, i)
@@ -262,7 +331,7 @@ class Dataset:
         :return:
         {
             "item": DatasetItem,
-            "annotations": List[Union[BoxAnnotation, PolygonAnnotation]],
+            "annotations": List[Union[BoxAnnotation, PolygonAnnotation, CuboidAnnotation, SegmentationAnnotation]],
         }
         """
         response = self._client.dataitem_ref_id(self.id, reference_id)
@@ -275,7 +344,7 @@ class Dataset:
         :return:
         {
             "item": DatasetItem,
-            "annotations": List[Union[BoxAnnotation, PolygonAnnotation]],
+            "annotations": List[Union[BoxAnnotation, PolygonAnnotation, CuboidAnnotation, SegmentationAnnotation]],
         }
         """
         response = self._client.dataitem_loc(self.id, dataset_item_id)
@@ -321,12 +390,23 @@ class Dataset:
     def list_autotags(self):
         return self._client.list_autotags(self.id)
 
-    def create_custom_index(self, embeddings_url: str):
-        return self._client.create_custom_index(self.id, embeddings_url)
+    def create_custom_index(self, embeddings_urls: list, embedding_dim: int):
+        return self._client.create_custom_index(
+            self.id,
+            embeddings_urls,
+            embedding_dim,
+        )
 
     def delete_custom_index(self):
         return self._client.delete_custom_index(self.id)
 
+    def set_continuous_indexing(self, enable: bool = True):
+        return self._client.set_continuous_indexing(self.id, enable)
+
+    def create_image_index(self):
+        response = self._client.create_image_index(self.id)
+        return AsyncJob.from_json(response, self._client)
+
     def check_index_status(self, job_id: str):
         return self._client.check_index_status(job_id)
 
@@ -349,3 +429,29 @@ class Dataset:
             requests_command=requests.get,
         )
         return convert_export_payload(api_payload[EXPORTED_ROWS])
+
+    def export_embeddings(
+        self,
+    ) -> List[Dict[str, Union[str, List[float]]]]:
+        """Returns a pd.Dataframe-ready format of dataset embeddings.
+
+        Returns:
+            A list, where each item is a dict with two keys representing a row
+            in the dataset.
+            * One value in the dict is the reference id
+            * The other value is a list of the embedding values
+        """
+        api_payload = self._client.make_request(
+            payload=None,
+            route=f"dataset/{self.id}/embeddings",
+            requests_command=requests.get,
+        )
+        return api_payload
+
+    def delete_annotations(
+        self, reference_ids: list = None, keep_history=False
+    ):
+        response = self._client.delete_annotations(
+            self.id, reference_ids, keep_history
+        )
+        return AsyncJob.from_json(response, self._client)
